@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -52,7 +52,7 @@ def _generateCallCodePosOnly(
     # TODO: Not yet specialized for method calls.
     # assert called_attribute_name is None
 
-    call_args = expression.getCallArgs()
+    call_args = expression.subnode_args
 
     if call_args is None or call_args.isExpressionConstantRef():
         context.setCurrentSourceCodeReference(expression.getCompatibleSourceReference())
@@ -226,9 +226,9 @@ def generateCallCode(to_name, expression, emit, context):
     # optimized code, constant, with and without positional or keyword arguments
     # each, so there is lots of branches involved.
 
-    called = expression.getCalled()
-    call_kw = expression.getCallKw()
-    call_args = expression.getCallArgs()
+    called = expression.subnode_called
+    call_kw = expression.subnode_kwargs
+    call_args = expression.subnode_args
 
     # TODO: Make this work for all cases. Currently, the method calls that do
     # a combined lookup and call, do a re-ordering of things, and therefore it
@@ -276,7 +276,7 @@ def generateCallCode(to_name, expression, emit, context):
                 context=context,
             )
         else:
-            call_args = expression.getCallArgs()
+            call_args = expression.subnode_args
 
             if call_args is None or call_args.isExpressionConstantTupleEmptyRef():
                 _generateCallCodeKwOnly(
@@ -376,17 +376,21 @@ def _getInstanceCallCodePosArgsQuick(
         emit(
             """\
 {
-    PyObject *call_args[] = {%s};
-    %s = CALL_METHOD_WITH_ARGS%d(%s, %s, call_args);
+    PyObject *call_args[] = {%(call_args)s};
+    %(to_name)s = CALL_METHOD_WITH_ARGS%(arg_size)d(
+        %(called_name)s,
+        %(called_attribute_name)s,
+        call_args
+    );
 }
 """
-            % (
-                ", ".join(str(arg_name) for arg_name in arg_names),
-                to_name,
-                arg_size,
-                called_name,
-                called_attribute_name,
-            )
+            % {
+                "call_args": ", ".join(str(arg_name) for arg_name in arg_names),
+                "to_name": to_name,
+                "arg_size": arg_size,
+                "called_name": called_name,
+                "called_attribute_name": called_attribute_name,
+            }
         )
 
     getErrorExitCode(
@@ -466,9 +470,19 @@ def _getInstanceCallCodeFromTuple(
 
     emit(
         """\
-%s = CALL_METHOD_WITH_ARGS%d(%s, %s, &PyTuple_GET_ITEM(%s, 0));
+%(to_name)s = CALL_METHOD_WITH_ARGS%(arg_size)d(
+    %(called_name)s,
+    %(called_attribute_name)s,
+    &PyTuple_GET_ITEM(%(arg_tuple)s, 0)
+);
 """
-        % (to_name, arg_size, called_name, called_attribute_name, arg_tuple)
+        % {
+            "to_name": to_name,
+            "arg_size": arg_size,
+            "called_name": called_name,
+            "called_attribute_name": called_attribute_name,
+            "arg_tuple": arg_tuple,
+        }
     )
 
     getErrorExitCode(
