@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,6 +24,7 @@ This is about collecting these constraints and to manage them.
 """
 
 import contextlib
+from contextlib import contextmanager
 
 from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
@@ -35,7 +36,11 @@ from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
 from nuitka.nodes.shapes.StandardShapes import tshape_uninit
 from nuitka.tree.SourceReading import readSourceLine
 from nuitka.utils.FileOperations import relpath
-from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.utils.InstanceCounters import (
+    counted_del,
+    counted_init,
+    isCountingInstances,
+)
 from nuitka.utils.ModuleNames import ModuleName
 
 from .ValueTraces import (
@@ -53,7 +58,27 @@ from .ValueTraces import (
 signalChange = None
 
 
+@contextmanager
+def withChangeIndicationsTo(signal_change):
+    """ Decide where change indications should go to."""
+
+    global signalChange  # Singleton, pylint: disable=global-statement
+
+    old = signalChange
+    signalChange = signal_change
+    yield
+    signalChange = old
+
+
 class CollectionStartpointMixin(object):
+    """Mixin to use in start points of collections.
+
+    These are modules, functions, etc. typically entry points.
+    """
+
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
     # Many things are traced
 
     def __init__(self):
@@ -157,6 +182,16 @@ class CollectionStartpointMixin(object):
 
         return result
 
+    def hasEmptyTraces(self, variable):
+        # TODO: Combine these steps into one for performance gains.
+        traces = self.getVariableTraces(variable)
+        return areEmptyTraces(traces)
+
+    def hasReadOnlyTraces(self, variable):
+        # TODO: Combine these steps into one for performance gains.
+        traces = self.getVariableTraces(variable)
+        return areReadOnlyTraces(traces)
+
     def getVariableTracesAll(self):
         return self.variable_traces
 
@@ -274,7 +309,10 @@ class TraceCollectionBase(object):
     They are kept for "variable" and versions.
     """
 
-    __del__ = counted_del()
+    __slots__ = ("owner", "parent", "name", "value_states", "variable_actives")
+
+    if isCountingInstances():
+        __del__ = counted_del()
 
     @counted_init
     def __init__(self, owner, name, parent):
@@ -374,6 +412,19 @@ class TraceCollectionBase(object):
     def onUsedModule(self, module_name, module_relpath):
         return self.parent.onUsedModule(module_name, module_relpath)
 
+    def onUsedFunction(self, function_body):
+        owning_module = function_body.getParentModule()
+
+        # Make sure the owning module is added to the used set. This is most
+        # important for helper functions, or modules, which otherwise have
+        # become unused.
+        addUsedModule(owning_module)
+
+        needs_visit = owning_module.addUsedFunction(function_body)
+
+        if needs_visit:
+            function_body.computeFunctionRaw(self)
+
     @staticmethod
     def mustAlias(a, b):
         if a.isExpressionVariableRef() and b.isExpressionVariableRef():
@@ -390,9 +441,6 @@ class TraceCollectionBase(object):
 
         return False
 
-    def removeKnowledge(self, node):
-        pass
-
     def onControlFlowEscape(self, node):
         # TODO: One day, we should trace which nodes exactly cause a variable
         # to be considered escaped, pylint: disable=unused-argument
@@ -407,6 +455,14 @@ class TraceCollectionBase(object):
                 if variable.hasAccessesOutsideOf(self.owner) is not False:
                     self.markActiveVariableAsEscaped(variable)
 
+    def removeKnowledge(self, node):
+        if node.isExpressionVariableRef():
+            self.markActiveVariableAsEscaped(node.variable)
+
+    def onValueEscapeStr(self, node):
+        # TODO: We can ignore these for now.
+        pass
+
     def removeAllKnowledge(self):
         self.markActiveVariablesAsUnknown()
 
@@ -470,9 +526,7 @@ class TraceCollectionBase(object):
         return result
 
     def onVariableContentEscapes(self, variable):
-        # TODO: Make use of this information, when we trace lists, dicts, etc.
-        # self.getVariableCurrentTrace(variable).onValueEscape()
-        pass
+        self.markActiveVariableAsEscaped(variable)
 
     def onExpression(self, expression, allow_none=False):
         if expression is None and allow_none:
@@ -574,19 +628,63 @@ class TraceCollectionBase(object):
         alternative branches to both change this collection.
         """
 
-        # Refuse to do stupid work
-        if collection_yes is None and collection_no is None:
-            return None
-        elif collection_yes is None or collection_no is None:
+        # Many branches due to inlining the actual merge and preparing it
+        # pylint: disable=too-many-branches
+
+        if collection_yes is None:
+            if collection_no is not None:
+                # Handle one branch case, we need to merge versions backwards as
+                # they may make themselves obsolete.
+                collection1 = self
+                collection2 = collection_no
+            else:
+                # Refuse to do stupid work
+                return
+        elif collection_no is None:
             # Handle one branch case, we need to merge versions backwards as
             # they may make themselves obsolete.
-            return self.mergeMultipleBranches(
-                collections=(self, collection_yes or collection_no)
-            )
+            collection1 = self
+            collection2 = collection_yes
         else:
-            return self.mergeMultipleBranches(
-                collections=(collection_yes, collection_no)
-            )
+            # Handle two branch case, they may or may not do the same things.
+            collection1 = collection_yes
+            collection2 = collection_no
+
+        variable_versions = {}
+
+        for variable, version in iterItems(collection1.variable_actives):
+            variable_versions[variable] = version
+
+        for variable, version in iterItems(collection2.variable_actives):
+            if variable not in variable_versions:
+                variable_versions[variable] = 0, version
+            else:
+                other = variable_versions[variable]
+
+                if other != version:
+                    variable_versions[variable] = other, version
+                else:
+                    variable_versions[variable] = other
+
+        for variable in variable_versions:
+            if variable not in collection2.variable_actives:
+                variable_versions[variable] = variable_versions[variable], 0
+
+        self.variable_actives = {}
+
+        for variable, versions in iterItems(variable_versions):
+            if type(versions) is tuple:
+                version = self.addVariableMergeMultipleTrace(
+                    variable=variable,
+                    traces=(
+                        self.getVariableTrace(variable, versions[0]),
+                        self.getVariableTrace(variable, versions[1]),
+                    ),
+                )
+            else:
+                version = versions
+
+            self.markCurrentVariableTrace(variable, version)
 
     def mergeMultipleBranches(self, collections):
         assert collections
@@ -613,8 +711,6 @@ class TraceCollectionBase(object):
 
         self.variable_actives = {}
 
-        #         merge_traces = None
-
         for variable, versions in iterItems(variable_versions):
             if len(versions) == 1:
                 (version,) = versions
@@ -626,17 +722,8 @@ class TraceCollectionBase(object):
                     ),
                 )
 
-            #                 if merge_traces is None:
-            #                     merge_traces = [trace_merge]
-            #                 else:
-            #                     merge_traces.append(trace_merge)
-
             self.markCurrentVariableTrace(variable, version)
 
-        # Return "None", or turn the list into a tuple for memory savings.
-
-    #         return merge_traces and tuple(merge_traces)
-
     def replaceBranch(self, collection_replace):
         self.variable_actives.update(collection_replace.variable_actives)
         collection_replace.variable_actives = None
@@ -692,7 +779,10 @@ class TraceCollectionBase(object):
 
     def getCompileTimeComputationResult(self, node, computation, description):
         new_node, change_tags, message = getComputationResult(
-            node=node, computation=computation, description=description
+            node=node,
+            computation=computation,
+            description=description,
+            user_provided=False,
         )
 
         if change_tags == "new_raise":
@@ -720,9 +810,12 @@ class TraceCollectionBase(object):
 
 
 class TraceCollectionBranch(TraceCollectionBase):
+    __slots__ = ()
+
     def __init__(self, name, parent):
         TraceCollectionBase.__init__(self, owner=parent.owner, name=name, parent=parent)
 
+        # Detach from others
         self.variable_actives = dict(parent.variable_actives)
 
     def computeBranch(self, branch):
@@ -759,6 +852,16 @@ class TraceCollectionBranch(TraceCollectionBase):
 
 
 class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
+    __slots__ = (
+        "variable_versions",
+        "variable_traces",
+        "break_collections",
+        "continue_collections",
+        "return_collections",
+        "exception_collections",
+        "outline_functions",
+    )
+
     def __init__(self, parent, function_body):
         assert (
             function_body.isExpressionFunctionBody()
@@ -796,7 +899,34 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
                 function_body.locals_scope = None
 
 
+class TraceCollectionPureFunction(TraceCollectionFunction):
+    """Pure functions don't feed their parent."""
+
+    __slots__ = ("used_functions",)
+
+    def __init__(self, function_body):
+        TraceCollectionFunction.__init__(self, parent=None, function_body=function_body)
+
+        self.used_functions = OrderedSet()
+
+    def getUsedFunctions(self):
+        return self.used_functions
+
+    def onUsedFunction(self, function_body):
+        self.used_functions.add(function_body)
+
+
 class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
+    __slots__ = (
+        "variable_versions",
+        "variable_traces",
+        "break_collections",
+        "continue_collections",
+        "return_collections",
+        "exception_collections",
+        "outline_functions",
+    )
+
     def __init__(self, module):
         assert module.isCompiledPythonModule(), module
 
@@ -817,3 +947,70 @@ class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
 
         module = getImportedModuleByNameAndPath(module_name, module_relpath)
         addUsedModule(module)
+
+
+# TODO: This should not exist, but be part of decision at the time these are collected.
+def areEmptyTraces(variable_traces):
+    """Do these traces contain any writes or accesses."""
+    # Many cases immediately return, that is how we do it here,
+    # pylint: disable=too-many-return-statements
+
+    for variable_trace in variable_traces:
+        if variable_trace.isAssignTrace():
+            return False
+        elif variable_trace.isInitTrace():
+            return False
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being removed.
+
+            return False
+        elif variable_trace.isUninitTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isUnknownTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isMergeTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isLoopTrace():
+            return False
+        else:
+            assert False, variable_trace
+
+    return True
+
+
+def areReadOnlyTraces(variable_traces):
+    """Do these traces contain any writes."""
+
+    # Many cases immediately return, that is how we do it here,
+    for variable_trace in variable_traces:
+        if variable_trace.isAssignTrace():
+            return False
+        elif variable_trace.isInitTrace():
+            pass
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being not released.
+
+            return False
+        elif variable_trace.isUninitTrace():
+            pass
+        elif variable_trace.isUnknownTrace():
+            return False
+        elif variable_trace.isMergeTrace():
+            pass
+        elif variable_trace.isLoopTrace():
+            pass
+        else:
+            assert False, variable_trace
+
+    return True
