@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -103,7 +103,7 @@ def encodeNonAscii(var_name):
     For Python3, unicode identifiers can be used, but these are not
     possible in C, so we need to replace them.
     """
-    if python_version < 300:
+    if python_version < 0x300:
         return var_name
     else:
         # Using a escaping here, because that makes it safe in terms of not
@@ -116,17 +116,5 @@ def encodeNonAscii(var_name):
         return var_name.replace("&#", "$$").replace(";", "")
 
 
-def isExecutableCommand(command):
-    path = os.environ["PATH"]
-
-    suffixes = (".exe",) if getOS() == "Windows" else ("",)
-
-    for part in path.split(os.pathsep):
-        if not part:
-            continue
-
-        for suffix in suffixes:
-            if os.path.isfile(os.path.join(part, command + suffix)):
-                return True
-
-    return False
+def hasOnefileSupportedOS():
+    return getOS() in ("Linux", "Windows")
